import datetime
import platform
from pathlib import Path

import matplotlib.pyplot as plt
import pandas as pd
from matplotlib.patches import Circle

from gameart.utils import utils


def _draw_mouse_tracks(csv_file_path: str, output_folder: str = "") -> None:
    """
    Draws a matlab figure with the recorded mouse movement displayed as a
    x-y-diagram. The movement will be drawn with lines and the mouse
    standstill positions will be drawn as circles, which vary in size
    according to the standstill duration on each position.
    The picture will be saved to the output folder.

    Args:
        csv_file_path (str): argument to provide path to csv file.
            If csv_file_path is not specified, csv file will be searched in
            the current working directory
        output_folder (str, optional): argument to provide an output path.
            If output_folder is not specified, the output image will be stored
            in the current working directory
    """
    if output_folder == "":
        output_path = Path(".")
    else:
        output_path = Path(output_folder)

    csv_file = Path(csv_file_path)
    data_frame_mouse_movement = pd.read_csv(csv_file, index_col=0)

    if platform.system() == "Windows":
        _, ax = plt.subplots(
            figsize=(
                utils._get_screensize()[0] / 100,
                utils._get_screensize()[1] / 100,
            )
        )
    else:
        screensize_default = (1920 / 100, 1080 / 100)
        _, ax = plt.subplots(figsize=screensize_default)

    x_prev, y_prev = None, None
    count_identical_rows = 0
    radius = 10
    gray_level = 0

    for _, row in data_frame_mouse_movement.iterrows():
        x, y = row["x-position"], row["y-position"]

        if x == x_prev and y == y_prev:
            count_identical_rows += 1
        else:
            if count_identical_rows >= 3:
                # Adjust the radius based on the duration of standstill
                radius = min(100, 10 + count_identical_rows)

                color = str(gray_level / 10)

                # Draw a circle with adjusted radius
                circle = Circle((x, y), radius, color=color)
                ax.add_patch(circle)

                gray_level = (gray_level + 1) % 10

            count_identical_rows = 0

        if x_prev is not None and y_prev is not None:
            # Draw a line for mouse movement
            ax.plot([x_prev, x], [y_prev, y], color="black", linewidth=0.5)

        x_prev, y_prev = x, y

    ax.set_aspect("equal", adjustable="box")
    plt.axis("off")

    output_folder_name = "gameart_images"
    utils._create_output_folder(output_path, output_folder_name)

    current_time = datetime.datetime.now()
    formatted_time = current_time.strftime("%Y-%m-%d_%H-%M-%S")
    file_name = f"gameart_{formatted_time}"

    try:
        plt.savefig(f"{output_path}/{output_folder_name}/{file_name}.png")
    except FileExistsError as e:
        print("Error: %s - %s." % (e.filename, e.strerror))



from unittest.mock import Mock, patch

import matplotlib.pyplot  # noqa: F401
import pandas

from gameart.draw import art
from gameart.utils import utils  # noqa: F401


def test_draw_mouse_tracks():
    """
    Case 1: Mock external dependencies, check if expected values are returned
    and verify that matplotlib.pyplot.show() was called to display the plot.
    """
    mock_csv_file = "/mock/csv/file.csv"
    mock_data_frame = pandas.DataFrame(
        {
            "x-position": [1009, 1004, 1003, 1003],
            "y-position": [97, 106, 106, 107],
        }
    )
    mock_screensize = Mock(return_value=(1920, 1080))
    mock_output_folder = Mock("/mock/git/root/path/gameart_images")

    with patch("pandas.read_csv", return_value=mock_data_frame), patch(
        "gameart.utils.utils._get_screensize", mock_screensize
    ), patch(
        "gameart.utils.utils._create_output_folder", mock_output_folder
    ), patch(
        "matplotlib.pyplot.savefig"
    ) as mock_save:
        art._draw_mouse_tracks(mock_csv_file)

    assert mock_save.called


def test_draw_mouse_tracks_output_folder():
    """
    Case 2: Mock external dependencies, check if expected values are returned
    and verify that matplotlib.pyplot.show() was called to display the plot.
    Specify output folder path.
    """
    mock_csv_file = "/mock/csv/file.csv"
    mock_data_frame = pandas.DataFrame(
        {
            "x-position": [1009, 1004, 1003, 1003],
            "y-position": [97, 106, 106, 107],
        }
    )
    mock_screensize = Mock(return_value=(1920, 1080))
    output_folder = "/mock/output/folder/"
    mock_output_folder = Mock(f"{output_folder}/gameart_images")

    with patch("pandas.read_csv", return_value=mock_data_frame), patch(
        "gameart.utils.utils._get_screensize", mock_screensize
    ), patch(
        "gameart.utils.utils._create_output_folder", mock_output_folder
    ), patch(
        "matplotlib.pyplot.savefig"
    ) as mock_save:
        art._draw_mouse_tracks(mock_csv_file, output_folder)

    assert mock_save.called



import logging

logging.basicConfig(level=logging.INFO)


def record_mouse() -> None:
    """
    Public function to start recording of mouse input
    """
    from gameart.inputs import mousetracker

    mousetracker._record_mouse()


def draw_mouse_tracks(csv_file_path: str, output_folder: str = "") -> None:
    """
    Public function to draw mousetracking-art based on csv file input
    which was generated by the method 'record_mouse'

    Args:
        csv_file_path (str): argument to provide path to csv file.
        output_folder (str, optional): argument to provide an output path.
    """
    from gameart.draw import art

    art._draw_mouse_tracks(csv_file_path, output_folder=output_folder)
